/*
 * Vifir - An electronic flight bag wannabe utility for aircraft pilots
 * Copyright (C) 2009 David Ludlow
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * dave@adsllc.com, PO Box 13070, Toledo, OH 43623
 */

#ifndef CONFIG_H_
#define CONFIG_H_

#include <string>
#include <list>

#define CONFIG_DEFAULT_ConfigVer 1
#define CONFIG_DEFAULT_GPSSource gpsd
#define CONFIG_DEFAULT_SerialPortDevice "/dev/ttyS0"
#define CONFIG_DEFAULT_SerialPortBaud 4800
#define CONFIG_DEFAULT_AutoStartgpsd true
#define CONFIG_DEFAULT_AutoActivateGPS true
#define CONFIG_DEFAULT_PDFViewer "evince %s"
#define CONFIG_DEFAULT_ImageViewer "eog %s"
#define CONFIG_DEFAULT_SaveConfigOnExit true
#define CONFIG_DEFAULT_CenterOnGPS true
#define CONFIG_DEFAULT_gpsdCmd "gpsd /dev/ttyS0"
#define CONFIG_DEFAULT_gpsdHost "127.0.0.1"
#define CONFIG_DEFAULT_gpsdPort "2947"
#define CONFIG_DEFAULT_GPSUpdateFrequency 2000
#define CONFIG_DEFAULT_PackageDataDir PACKAGEDATADIR
#define CONFIG_DEFAULT_UserDataDir USERDATADIR
#define CONFIG_DEFAULT_DataStoreDir DATASTOREDIR
#define CONFIG_DEFAULT_METARSourceURL "ftp://tgftp.nws.noaa.gov/data/observations/metar/cycles/"
#define CONFIG_DEFAULT_AutoSelectChart true
#define CONFIG_DEFAULT_GPS_SimLon -83.4822614
#define CONFIG_DEFAULT_GPS_SimLat 41.5648719
#define CONFIG_DEFAULT_GPS_SimTrk 0.0
#define CONFIG_DEFAULT_GPS_SimSpd 100.0
#define CONFIG_DEFAULT_GPS_SimAlt 5000.0
#define CONFIG_DEFAULT_Disable_SS true
#define CONFIG_DEFAULT_Disable_PM true
#define CONFIG_DEFAULT_ChartScale 1.0
#define CONFIG_DEFAULT_ChartXOffset 0
#define CONFIG_DEFAULT_ChartYOffset 0
#define CONFIG_DEFAULT_OpenChartFile ""
#define CONFIG_DEFAULT_OpenRouteFile ""
#define CONFIG_DEFAULT_OpenAircraftFile ""
#define CONFIG_DEFAULT_ShowWelcomeOnStartup true
#define CONFIG_DEFAULT_METARHistory 2
#define CONFIG_DEFAULT_ShapeFile (std::string(PACKAGEDATADIR)+std::string("/us_states.shp"))

#ifndef WIN32
 #define CONFIG_DEFAULT_dTPPImportDir "/media/d-TPP/xml"
 #define CONFIG_DEFAULT_SectionalsImportDir "/media"
 #define CONFIG_DEFAULT_DigitalAeroInfoImportDir "/media"
 #define CONFIG_DEFAULT_NationalFlightDBImportDir "/media/NationalFlightDatabase"
 #define CONFIG_DEFAULT_XPlane900DataImportDir "/media"
#else
 #define CONFIG_DEFAULT_dTPPImportDir "~"
 #define CONFIG_DEFAULT_SectionalsImportDir "~"
 #define CONFIG_DEFAULT_DigitalAeroInfoImportDir "~"
 #define CONFIG_DEFAULT_NationalFlightDBImportDir "~"
 #define CONFIG_DEFAULT_XPlane900DataImportDir "~"
#endif

class Config
{
public:
	enum GPSSource_enum {
		gpsd,
		Serial,
		Simulated
	};

	Config();
	virtual ~Config();

	bool Load();
	bool Save();
	void SetDefaults();

	int GetConfigVer();
	void SetConfigVer(int value=CONFIG_DEFAULT_ConfigVer);
	std::string GetSerialPortDevice();
	void SetSerialPortDevice(std::string value=CONFIG_DEFAULT_SerialPortDevice);
	int GetSerialPortBaud();
	void SetSerialPortbaud(int value=CONFIG_DEFAULT_SerialPortBaud);
	bool GetAutoActivateGPS();
	void SetAutoActivateGPS(bool value=CONFIG_DEFAULT_AutoActivateGPS);
	bool GetAutoStartgpsd();
	void SetAutoStartgpsd(bool value=CONFIG_DEFAULT_AutoStartgpsd);
	std::string GetPDFViewer();
	void SetPDFViewer(std::string value=CONFIG_DEFAULT_PDFViewer);
	std::string GetImageViewer();
	void SetImageViewer(std::string value=CONFIG_DEFAULT_ImageViewer);
	bool GetDisableSS();
	void SetDisableSS(bool value=CONFIG_DEFAULT_Disable_SS);
	bool GetDisablePM();
	void SetDisablePM(bool value=CONFIG_DEFAULT_Disable_PM);
	bool GetSaveConfigOnExit();
	void SetSaveConfigOnExit(bool value=CONFIG_DEFAULT_SaveConfigOnExit);
	std::string GetUserDataDir();
	void SetUserDataDir(std::string value);
	std::string GetDataStoreDir();
	void SetDataStoreDir(std::string value);
	std::string GetPackageDataDir();
	std::string GetMETARSourceURL();
	void SetMETARSourceURL(std::string value);
	void SetCenterOnGPS(bool value);
	bool GetCenterOnGPS();
	void SetgpsdCmd(std::string value);
	std::string GetgpsdCmd();
	void SetgpsdHost(std::string value);
	std::string GetgpsdHost();
	void SetgpsdPort(std::string value);
	std::string GetgpsdPort();
	void SetGPSUpdateFrequency(unsigned int value);
	time_t GetGPSUpdateFrequency();
	void SetdTPPImportDir(std::string value);
	std::string GetdTPPImportDir();
	void SetSectionalsImportDir(std::string value);
	std::string GetSectionalsImportDir();
	void SetDigitalAeroInfoImportDir(std::string value);
	std::string GetDigitalAeroInfoImportDir();
	void SetNationalFlightDBImportDir(std::string value);
	std::string GetNationalFlightDBImportDir();
	void SetXPlane900DataImportDir(std::string value);
	std::string GetXPlane900DataImportDir();
	void SetAutoSelectChart(bool value);
	bool GetAutoSelectChart();
	void SetGPSSimLon(double value);
	double GetGPSSimLon();
	void SetGPSSimLat(double value);
	double GetGPSSimLat();
	void SetGPSSimTrk(double value);
	double GetGPSSimTrk();
	void SetGPSSimSpd(double value);
	double GetGPSSimSpd();
	void SetGPSSimAlt(double value);
	double GetGPSSimAlt();
	void SetGPSSource(enum Config::GPSSource_enum value);
	bool SetGPSSource(std::string value);
	enum Config::GPSSource_enum GetGPSSource();
	void SetChartScale(double value=CONFIG_DEFAULT_ChartScale);
	double GetChartScale();
	void SetChartXOffset(int value);
	int GetChartXOffset();
	void SetChartYOffset(int value);
	int GetChartYOffset();
	void SetOpenChartFile(std::string value=CONFIG_DEFAULT_OpenChartFile);
	std::string GetOpenChartFile();
	void SetOpenRouteFile(std::string value=CONFIG_DEFAULT_OpenRouteFile);
	std::string GetOpenRouteFile();
	void SetOpenAircraftFile(std::string value=CONFIG_DEFAULT_OpenRouteFile);
	std::string GetOpenAircraftFile();
	void SetShowWelcomeOnStartup(bool value);
	bool GetShowWelcomeOnStartup();
	void SetMETARHistory(int value);
	int GetMETARHistory();
	void SetShapeFile(std::string value=CONFIG_DEFAULT_ShapeFile);
	std::string GetShapeFile();

private:
	int ConfigVer;
	enum GPSSource_enum GPSSource;
	std::string SerialPortDevice;
	int SerialPortBaud;
	bool AutoActivateGPS;
	bool AutoStartgpsd;
	std::string PDFViewer;
	std::string ImageViewer;
	bool SaveConfigOnExit;
	std::string PackageDataDir;
	std::string UserDataDir;
	bool CenterOnGPS;
	std::string gpsdCmd;
	std::string gpsdHost;
	std::string gpsdPort;
	time_t GPSUpdateFrequency;
	std::string dTPPImportDir;
	std::string SectionalsImportDir;
	std::string DigitalAeroInfoImportDir;
	std::string NationalFlightDBImportDir;
	std::string XPlane900DataImportDir;
	std::string METARSourceURL;
	bool AutoSelectChart;
	double GPSSimLon;
	double GPSSimLat;
	double GPSSimTrk;
	double GPSSimSpd;
	double GPSSimAlt;
	std::string DataStoreDir;
	bool DisableSS,DisablePM;
	double ChartScale;
	int ChartXOffset;
	int ChartYOffset;
	std::string OpenChartFile;
	std::string OpenRouteFile;
	std::string OpenAircraftFile;
	bool ShowWelcomeOnStartup;
	int METARHistory;
	std::string ShapeFile;

	bool MkBool(std::string value);
};

#endif /*CONFIG_H_*/
