/*
 * Vifir - An electronic flight bag wannabe utility for aircraft pilots
 * Copyright (C) 2009 David Ludlow
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * dave@adsllc.com, PO Box 13070, Toledo, OH 43623
 */

#define __STDC_LIMIT_MACROS
#include <stdint.h>
#include <math.h>
#include <cstring>
#include "GeoChart.h"

GeoChart::GeoChart()
{
	EastBounding=WestBounding=NorthBounding=SouthBounding=0.0;
	GeoReferenced=false;
	Type=TYPE_SECTIONAL;
}

void GeoChart::SetFileName(std::string FileName) {
	FileName=FileName.substr(FileName.find_last_of("/\\")+1);	/* Strip off the path */
	this->FileName=FileName;
	Name=FileName;
	Name.erase(Name.find_last_of('.'));	/* Strip off the file extension */
}

std::string GeoChart::GetFileName() {
	return FileName;
}

void GeoChart::SetName(std::string Name) {
	this->Name=Name;
}

std::string GeoChart::GetName() {
	return Name;
}

void GeoChart::SetBounding(double North, double East, double South, double West) {
	NorthBounding=North;
	EastBounding=East;
	SouthBounding=South;
	WestBounding=West;
}

double GeoChart::GetNorthBounding() {
	return NorthBounding;
}

double GeoChart::GetEastBounding() {
	return EastBounding;
}

double GeoChart::GetSouthBounding() {
	return SouthBounding;
}

double GeoChart::GetWestBounding() {
	return WestBounding;
}

void GeoChart::SetGeoReferenced(bool GeoReferenced) {
	this->GeoReferenced=GeoReferenced;
}

bool GeoChart::GetGeoReferenced() {
	return GeoReferenced;
}

void GeoChart::SetType(Type_enum Type) {
	this->Type=Type;
}

GeoChart::Type_enum GeoChart::GetType() {
	return Type;
}

bool GeoChart::MakeBinary(struct GeoChart_Binary_struct *BinaryData) {
	double NorthBoundingx1000000,EastBoundingx1000000,SouthBoundingx1000000,WestBoundingx1000000;

	NorthBoundingx1000000=NorthBounding*1000000.0;
	EastBoundingx1000000=EastBounding*1000000.0;
	SouthBoundingx1000000=SouthBounding*1000000.0;
	WestBoundingx1000000=WestBounding*1000000.0;

	/* Sanity checking */
	if (
		FileName.size()>sizeof(BinaryData->FileName) ||
		Name.size()>sizeof(BinaryData->Name) ||
		NorthBoundingx1000000<INT32_MIN || NorthBoundingx1000000>INT32_MAX ||
		EastBoundingx1000000<INT32_MIN || EastBoundingx1000000>INT32_MAX ||
		SouthBoundingx1000000<INT32_MIN || SouthBoundingx1000000>INT32_MAX ||
		WestBoundingx1000000<INT32_MIN || WestBoundingx1000000>INT32_MAX ||
		GeoReferenced>INT8_MAX ||
		Type>INT8_MAX
		) {
		return false;
	}

	strncpy(BinaryData->FileName,FileName.c_str(),sizeof(BinaryData->FileName));
	strncpy(BinaryData->Name,Name.c_str(),sizeof(BinaryData->Name));
	BinaryData->NorthBoundingx1000000=(int32_t)round(NorthBoundingx1000000);
	BinaryData->EastBoundingx1000000=(int32_t)round(EastBoundingx1000000);
	BinaryData->SouthBoundingx1000000=(int32_t)round(SouthBoundingx1000000);
	BinaryData->WestBoundingx1000000=(int32_t)round(WestBoundingx1000000);
	BinaryData->GeoReferenced=GeoReferenced;
	BinaryData->Type=Type;

	return true;
}

void GeoChart::SetFromBinary(struct GeoChart_Binary_struct *BinaryData) {
	FileName.assign(BinaryData->FileName,sizeof(BinaryData->FileName));
	Name.assign(BinaryData->Name,sizeof(BinaryData->Name));
	/* Reset the size to the actual number of characters */
	FileName=FileName.c_str();
	Name=Name.c_str();

	NorthBounding=BinaryData->NorthBoundingx1000000/1000000.0;
	EastBounding=BinaryData->EastBoundingx1000000/1000000.0;
	SouthBounding=BinaryData->SouthBoundingx1000000/1000000.0;
	WestBounding=BinaryData->WestBoundingx1000000/1000000.0;
	GeoReferenced=BinaryData->GeoReferenced;
	Type=(Type_enum)BinaryData->Type;
}

